@@ -49,6 +49,7 @@ void QMLScreen::setConfig(const KScreen::ConfigPtr &config)
 {
     qDeleteAll(m_outputMap);
     m_outputMap.clear();
+    m_manuallyMovedOutputs.clear();
     m_bottommost = m_leftmost = m_rightmost = m_topmost = nullptr;
     m_connectedOutputsCount = 0;
     m_enabledOutputsCount = 0;
@@ -175,7 +176,7 @@ QSize QMLScreen::maxScreenSize() const
 
 float QMLScreen::outputScale() const
 {
-    return 1.0 / 12.0;
+    return m_outputScale;
 }
 
 void QMLScreen::outputConnectedChanged()
@@ -225,6 +226,8 @@ void QMLScreen::qmlOutputMoved(QMLOutput *qmlOutput)
     if (qmlOutput->isCloneMode()) {
         return;
     }
+    if (!m_manuallyMovedOutputs.contains(qmlOutput))
+        m_manuallyMovedOutputs.append(qmlOutput);
 
     updateCornerOutputs();
 
@@ -304,9 +307,20 @@ void QMLScreen::updateCornerOutputs()
     }
 }
 
+void QMLScreen::setOutputScale(float scale)
+{
+    if (qFuzzyCompare(scale, m_outputScale))
+        return;
+    m_outputScale = scale;
+    emit outputScaleChanged();
+}
+
 void QMLScreen::updateOutputsPlacement()
 {
-    QSizeF activeScreenSize;
+    if (width() <= 0)
+        return;
+
+    QSizeF initialActiveScreenSize;
 
     Q_FOREACH (QQuickItem *item, childItems()) {
         QMLOutput *qmlOutput = qobject_cast<QMLOutput*>(item);
@@ -314,42 +328,57 @@ void QMLScreen::updateOutputsPlacement()
             continue;
         }
 
-        if (qmlOutput->outputX() + qmlOutput->currentOutputWidth() > activeScreenSize.width()) {
-            activeScreenSize.setWidth(qmlOutput->outputX() + qmlOutput->currentOutputWidth());
+        if (qmlOutput->outputX() + qmlOutput->currentOutputWidth() > initialActiveScreenSize.width()) {
+            initialActiveScreenSize.setWidth(qmlOutput->outputX() + qmlOutput->currentOutputWidth());
         }
-        if (qmlOutput->outputY() + qmlOutput->currentOutputHeight() > activeScreenSize.height()) {
-            activeScreenSize.setHeight(qmlOutput->outputY() + qmlOutput->currentOutputHeight());
+        if (qmlOutput->outputY() + qmlOutput->currentOutputHeight() > initialActiveScreenSize.height()) {
+            initialActiveScreenSize.setHeight(qmlOutput->outputY() + qmlOutput->currentOutputHeight());
         }
     }
 
-    activeScreenSize *= outputScale();
-
-    const QPointF offset((width() - activeScreenSize.width()) / 2.0,
-                         (height() - activeScreenSize.height()) / 2.0);
-
+    auto initialScale = outputScale();
+    auto scale = initialScale;
     qreal lastX = -1.0;
-    qreal lastY = -1.0;
-    Q_FOREACH (QQuickItem *item, childItems()) {
-        QMLOutput *qmlOutput = qobject_cast<QMLOutput*>(item);
-        if (!qmlOutput->output()->isConnected() || !qmlOutput->output()->isEnabled()) {
-            continue;
-        }
+    do {
+        auto activeScreenSize = initialActiveScreenSize * scale;
 
-        qmlOutput->blockSignals(true);
-        qmlOutput->setPosition(QPointF(offset.x() + (qmlOutput->outputX() * outputScale()),
-                          offset.y() + (qmlOutput->outputY() * outputScale())));
-        lastX = qMax(lastX, qmlOutput->position().x() + qmlOutput->size().width());
-        lastY = qMax(lastY, qmlOutput->position().y());
-        qmlOutput->blockSignals(false);
-    }
+        const QPointF offset((width() - activeScreenSize.width()) / 2.0,
+                             (height() - activeScreenSize.height()) / 2.0);
+
+        lastX = -1.0;
+        qreal lastY = -1.0;
+        Q_FOREACH (QQuickItem *item, childItems()) {
+            QMLOutput *qmlOutput = qobject_cast<QMLOutput*>(item);
+            if (!qmlOutput->output()->isConnected() || !qmlOutput->output()->isEnabled() ||
+                 m_manuallyMovedOutputs.contains(qmlOutput)) {
+                continue;
+            }
 
-    Q_FOREACH (QQuickItem *item, childItems()) {
-        QMLOutput *qmlOutput = qobject_cast<QMLOutput*>(item);
-        if (qmlOutput->output()->isConnected() && !qmlOutput->output()->isEnabled()) {
             qmlOutput->blockSignals(true);
-            qmlOutput->setPosition(QPointF(lastX, lastY));
-            lastX += qmlOutput->size().width();
+            qmlOutput->setPosition(QPointF(offset.x() + (qmlOutput->outputX() * scale),
+                              offset.y() + (qmlOutput->outputY() * scale)));
+            lastX = qMax(lastX, qmlOutput->position().x() + qmlOutput->width() / initialScale * scale);
+            lastY = qMax(lastY, qmlOutput->position().y());
             qmlOutput->blockSignals(false);
         }
-    }
+
+        Q_FOREACH (QQuickItem *item, childItems()) {
+            QMLOutput *qmlOutput = qobject_cast<QMLOutput*>(item);
+            if (qmlOutput->output()->isConnected() && !qmlOutput->output()->isEnabled() &&
+                !m_manuallyMovedOutputs.contains(qmlOutput)) {
+                qmlOutput->blockSignals(true);
+                qmlOutput->setPosition(QPointF(lastX, lastY));
+                lastX += qmlOutput->width() / initialScale * scale;
+                qmlOutput->blockSignals(false);
+            }
+        }
+        // calculate the scale dynamically, so all screens fit to the dialog
+        if (lastX > width()) {
+            scale *= 0.8;
+        }
+    } while (lastX > width());
+    // Use a timer to avoid binding loop on width()
+    QTimer::singleShot(0, this, [scale, this] {
+        setOutputScale(scale);
+    });
 }
