@@ -19,7 +19,6 @@
  *  along with this program; if not, write to the Free Software                      *
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA   *
  *************************************************************************************/
-
 #include "daemon.h"
 #include "serializer.h"
 #include "generator.h"
@@ -28,26 +27,22 @@
 #include "kscreen_daemon_debug.h"
 #include "osdmanager.h"
 
-#include <QTimer>
-#include <QAction>
-#include <QShortcut>
-#include <QLoggingCategory>
+#include <kscreen/log.h>
+#include <kscreen/output.h>
+#include <kscreen/configmonitor.h>
+#include <kscreen/getconfigoperation.h>
+#include <kscreen/setconfigoperation.h>
 
 #include <KLocalizedString>
 #include <KActionCollection>
 #include <KPluginFactory>
 #include <KGlobalAccel>
 
-#include <kscreen/log.h>
-#include <kscreen/config.h>
-#include <kscreen/output.h>
-#include <kscreen/configmonitor.h>
-#include <kscreen/getconfigoperation.h>
-#include <kscreen/setconfigoperation.h>
+#include <QTimer>
+#include <QAction>
+#include <QShortcut>
 
-K_PLUGIN_FACTORY_WITH_JSON(KScreenDaemonFactory,
-                           "kscreen.json",
-                           registerPlugin<KScreenDaemon>();)
+K_PLUGIN_CLASS_WITH_JSON(KScreenDaemon, "kscreen.json")
 
 KScreenDaemon::KScreenDaemon(QObject* parent, const QList< QVariant >& )
  : KDEDModule(parent)
@@ -59,26 +54,23 @@ KScreenDaemon::KScreenDaemon(QObject* parent, const QList< QVariant >& )
  
 {
     KScreen::Log::instance();
-    QMetaObject::invokeMethod(this, "requestConfig", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, "getInitialConfig", Qt::QueuedConnection);
 }
 
-void KScreenDaemon::requestConfig()
+void KScreenDaemon::getInitialConfig()
 {
     connect(new KScreen::GetConfigOperation, &KScreen::GetConfigOperation::finished,
-            this, &KScreenDaemon::configReady);
-}
+            this, [this](KScreen::ConfigOperation* op) {
+        if (op->hasError()) {
+            return;
+        }
 
-void KScreenDaemon::configReady(KScreen::ConfigOperation* op)
-{
-    if (op->hasError()) {
-        return;
-    }
+        m_monitoredConfig = qobject_cast<KScreen::GetConfigOperation*>(op)->config();
+        qCDebug(KSCREEN_KDED) << "Config" << m_monitoredConfig.data() << "is ready";
+        KScreen::ConfigMonitor::instance()->addConfig(m_monitoredConfig);
 
-    m_monitoredConfig = qobject_cast<KScreen::GetConfigOperation*>(op)->config();
-    qCDebug(KSCREEN_KDED) << "Config" << m_monitoredConfig.data() << "is ready";
-    KScreen::ConfigMonitor::instance()->addConfig(m_monitoredConfig);
-
-    init();
+        init();
+    });
 }
 
 KScreenDaemon::~KScreenDaemon()
@@ -108,7 +100,6 @@ void KScreenDaemon::init()
     m_lidClosedTimer->setSingleShot(true);
     connect(m_lidClosedTimer, &QTimer::timeout, this, &KScreenDaemon::lidClosedTimeout);
 
-
     connect(Device::self(), &Device::lidClosedChanged, this, &KScreenDaemon::lidClosedChanged);
     connect(Device::self(), &Device::resumingFromSuspend, this,
             [&]() {
@@ -125,7 +116,6 @@ void KScreenDaemon::init()
                 m_lidClosedTimer->stop();
             });
 
-
     connect(Generator::self(), &Generator::ready,
             this, &KScreenDaemon::applyConfig);
 
@@ -168,7 +158,7 @@ void KScreenDaemon::applyKnownConfig()
         Serializer::moveConfig(configId  + QLatin1String("_lidOpened"), configId);
     }
 
-    KScreen::ConfigPtr config = Serializer::config(m_monitoredConfig, configId);
+    KScreen::ConfigPtr config = Serializer::loadConfig(m_monitoredConfig, configId);
     // It's possible that the Serializer returned a nullptr
     if (!config || !KScreen::Config::canBeApplied(config, KScreen::Config::ValidityFlag::RequireAtLeastOneEnabledScreen)) {
         applyIdealConfig();
@@ -218,13 +208,11 @@ void KScreenDaemon::applyOsdAction(KScreen::OsdAction::Action action)
         doApplyConfig(Generator::self()->displaySwitch(Generator::Clone));
         return;
     }
-
     Q_UNREACHABLE();
 }
 
 void KScreenDaemon::applyIdealConfig()
 {
-
     if (m_monitoredConfig->connectedOutputs().count() < 2) {
         m_osdManager->hideOsd();
         doApplyConfig(Generator::self()->idealConfig(m_monitoredConfig));
@@ -236,7 +224,8 @@ void KScreenDaemon::applyIdealConfig()
     }
 }
 
-void logConfig(const KScreen::ConfigPtr &config) {
+void logConfig(const KScreen::ConfigPtr &config)
+{
     if (config) {
         foreach (auto o, config->outputs()) {
             if (o->isConnected()) {
@@ -252,13 +241,17 @@ void KScreenDaemon::configChanged()
     logConfig(m_monitoredConfig);
 
     // Modes may have changed, fix-up current mode id
+    bool changed = false;
     Q_FOREACH(const KScreen::OutputPtr &output, m_monitoredConfig->outputs()) {
-        if (output->isConnected() && output->isEnabled() && output->currentMode().isNull()) {
-            qCDebug(KSCREEN_KDED) << "Current mode" << output->currentModeId() << "invalid, setting preferred mode" << output->preferredModeId();
+        if (output->isConnected() && output->isEnabled() && (output->currentMode().isNull() || (output->followPreferredMode() && output->currentModeId() != output->preferredModeId()))) {
+            qCDebug(KSCREEN_KDED) << "Current mode was" << output->currentModeId() << ", setting preferred mode" << output->preferredModeId();
             output->setCurrentModeId(output->preferredModeId());
-            doApplyConfig(m_monitoredConfig);
+            changed = true;
         }
     }
+    if (changed) {
+        doApplyConfig(m_monitoredConfig);
+    }
 
     // Reset timer, delay the writeback
     if (!m_saveTimer) {
@@ -336,7 +329,7 @@ void KScreenDaemon::lidClosedChanged(bool lidIsClosed)
 
         const QString openConfigId = Serializer::configId(m_monitoredConfig) + QLatin1String("_lidOpened");
         if (Serializer::configExists(openConfigId)) {
-            const KScreen::ConfigPtr openedConfig = Serializer::config(m_monitoredConfig, openConfigId);
+            const KScreen::ConfigPtr openedConfig = Serializer::loadConfig(m_monitoredConfig, openConfigId);
             Serializer::removeConfig(openConfigId);
 
             doApplyConfig(openedConfig);
